/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import jreqtools.gadgets.*;

/**
* This is the abstract base class for all requesters.
* 
* @author	Markus Hillenbrand
*/

public abstract class AbstractRequester {
	public static final int CANCEL	= -1;
	public static final int OK			= 0;
	
	protected ActionListener		listener		= null;
	
	private	Dimension 				size 		= null;
	private	Point					location		= null;
	private	String					title			= null;
	private JComponent			parentC	= null;
	
	protected void addHelpButton(Container container)  {
		JButton help = new JButton("Help", UIManager.getIcon("jreqtools.helpIcon"));
		container.add(new Strut());
		container.add(Box.createHorizontalGlue());
		container.add(help);
		help.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				listener.actionPerformed(new ActionEvent(AbstractRequester.this, ActionEvent.ACTION_PERFORMED, "Help"));
			}
		});
	}

	public void addHelpButton(ActionListener l)  {
		listener = l;
	}

	protected abstract JDialog createJDialog(JFrame parent) ;

	public Point getLocation()  {
		return location == null ? new Point(-1,-1) : location;
	}

	protected abstract int getReturnValue() ;

	public Dimension getSize()  {
		return size == null ? new Dimension(0,0) : size;
	}

	public int request()  {
		return this.request(null);
	}

	public int request(JFrame parent)  {
		// den Dialog erstellen
		JDialog d = createJDialog(parent);
		
		// Den Titel
		if (title != null) d.setTitle(title);
		
		// Default-Werte
		d.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		
		// Die Gre setzen
		if (size != null) 
			{
			Dimension dim = d.getSize();
			if (dim.width > size.width) size.width = dim.width;
			if (dim.height > size.height) size.height = dim.height;
			d.setSize(size);
			}
		
		// Falls es eine Parent-Komponente gibt, dann Position berechnen
		if (parentC != null && parentC.isVisible())
			{
			Point loc = parentC.getLocationOnScreen();
			location = new Point(Math.max(0, loc.x-10), Math.max(0,loc.y-10));
			}
		
		// Wenn ntig, die Position berechnen (mitten auf dem Screen)
		if (location == null) 
			{
			Toolkit tk = d.getToolkit();
			Dimension ds = tk.getScreenSize();
			Dimension dd = (size != null) ? size : d.getSize();
			location = new Point( (ds.width - dd.width) / 2, (ds.height - dd.height) / 2 );
			}
		
		// Fehlerkorrektur bei der Fensterpositionierung
		String version = System.getProperty("java.fullversion");
		if (version != null && version.startsWith("J2RE 1.3.0 IBM build cx130-20010207"))
			{
			location.x -= 5;
			location.y -= 44;
			}
		
		// Den Dialog positionieren
		d.setLocation(location);
		
		// Den Dialog anzeigen
		d.setVisible(true);
		
		// Frs nchste mal Position und Gre holen
		location = d.getLocation();
		size = d.getSize();
		
		// Dispose aufrufen
		d.dispose();
		
		// Return Wert
		return getReturnValue();
	}

	public void setLocation(Point newlocation)  {
		location = (newlocation == null || newlocation.x == -1) ? null : newlocation;
	}

	public void setLocation(int x, int y)  {
		location = x == -1 ? null : new Point(x,y);
	}

	public void setParentComponent(JComponent c)  {
		parentC = c;
	}

	public void setSize(Dimension newsize)  {
		size = (newsize == null || newsize.width == 0) ? null : newsize;
	}

	public void setSize(int width, int height)  {
		size = width == 0 ? null : new Dimension(width, height);
	}

	public void setTitle(String newtitle)  {
		title = newtitle;
	}

}
