/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import jreqtools.gadgets.*;

public final class JReqTools {
	private static final 	ResourceBundle resources 	= ResourceBundle.getBundle("jreqtools.resources.jreqtools");
	private static final 	int				frameSpace	= UIManager.getInt("jreqtools.frameSpace");
	private static final 	int				objectSpace	= UIManager.getInt("jreqtools.objectSpace");
	private static final 	Border			border		= BorderFactory.createEmptyBorder(frameSpace,frameSpace,frameSpace,frameSpace);
	private static final 	Hashtable		icons		= new Hashtable();
	
	private final static class MessageDialog extends JDialog {
		JButton ok = new JButton(JReqTools.getResourceString("Ok"), UIManager.getIcon("jreqtools.okIcon"));
		
		public MessageDialog(Frame parent, String title, String message)  {
			super(parent, title, true);
			
			// Listener
			ok.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { setVisible(false); } });
			
			// Button erzeugen
			Box buttons = Box.createHorizontalBox();
			buttons.add(Box.createHorizontalGlue());
			buttons.add(ok);
			buttons.add(Box.createHorizontalGlue());
			
			// Nachricht erzeugen
			MessageLines ml = new MessageLines(message, MessageLines.LEFT);
			ml.setBorder(new BigBorder());
			ml.setOpaque(true);
			
			// Panel erzeugen
			JPanel p = new YFramePanel();
			p.add(Box.createVerticalGlue());
			p.add(ml);
			p.add(Box.createVerticalGlue());
			p.add(new Strut());
			p.add(buttons);
			
			// Farbe ndern
			Color c = ml.getBackground();
			float [] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
			ml.setBackground(new Color(Color.HSBtoRGB(hsb[0], hsb[1], (float)Math.max(0.0, hsb[2]-0.1))));
			
			// Alles hinzufgen
			setContentPane(p);
			getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
			pack();
			setResizable(false);
			 
			// Positionieren
			Dimension dim = getToolkit().getScreenSize();
			Dimension size = getSize();
			setLocation( (dim.width-size.width)/2, (dim.height-size.height)/2);
		}

		public void setVisible(boolean bool)  {
			if (bool) ok.requestFocus();
			super.setVisible(bool);
		}

	}

	private static class YesNoDialog extends JDialog {
		JButton 	yes 	= new JButton(JReqTools.getResourceString("Yes"), UIManager.getIcon("jreqtools.okIcon"));
		JButton 	no		= new JButton(JReqTools.getResourceString("No"), UIManager.getIcon("jreqtools.cancelIcon"));
		boolean		answer	= false;
		
		public YesNoDialog(JFrame parent, String title, String q, String y, String n)  {
			super(parent, title, true);
			
			// Gadgets einrichten
			yes.setText(y);
			no.setText(n);
			
			// Listener
			yes.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { answer = true; setVisible(false); } });
			no.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { answer = false; setVisible(false); } });
			
			// Button erzeugen
			Box buttons = Box.createHorizontalBox();
			buttons.add(yes);
			buttons.add(Box.createHorizontalGlue());
			buttons.add(new Strut());
			buttons.add(Box.createHorizontalGlue());
			buttons.add(no);
			
			// Nachricht erzeugen
			MessageLines ml = new MessageLines(q, MessageLines.LEFT);
			ml.setBorder(new BigBorder());
			ml.setOpaque(true);
			
			// Panel erzeugen
			JPanel p = new YFramePanel();
			p.add(Box.createVerticalGlue());
			p.add(ml);
			p.add(Box.createVerticalGlue());
			p.add(new Strut());
			p.add(buttons);
			
			// Farbe ndern
			Color c = ml.getBackground();
			float [] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
			ml.setBackground(new Color(Color.HSBtoRGB(hsb[0], hsb[1], (float)Math.max(0.0, hsb[2]-0.1))));
			
			// Alles hinzufgen
			setContentPane(p);
			getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
			pack();
			setResizable(false);
			 
			// Positionieren
			Dimension dim = getToolkit().getScreenSize();
			Dimension size = getSize();
			setLocation( (dim.width-size.width)/2, (dim.height-size.height)/2);
		}

		public boolean getAnswer()  {
			return answer;
		}

		public void setVisible(boolean bool)  {
			if (bool) no.requestFocus();
			super.setVisible(bool);
		}

	}

	private JReqTools()  {
	}

	static {
		if (UIManager.get("jreqtools.frameSpace") == null)	UIManager.put("jreqtools.frameSpace"	, new Integer(10));
		if (UIManager.get("jreqtools.objectSpace") == null)	UIManager.put("jreqtools.objectSpace"	, new Integer(5));
	}

	private static String createStringFromException(Throwable t)  {
		if (t == null) return "";
		
		try
			{
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw);
			t.printStackTrace(pw);
		
			StringBuffer buf = sw.getBuffer();
			if (buf == null) return "";
		
			for (int i=0; i<buf.length(); ++i)
				{
				if (buf.charAt(i) != '\t') continue;
				buf.setCharAt(i, ' ');
				buf.insert(i, "   ");
				i+=3;
				}
		
			return buf.toString();
			}
		catch (Throwable th) 
			{ 
		//	JReqTools.showError(getResourceString("InternalError") + " JReqTools.createStringFromException()");
			th.printStackTrace(System.err);
			return "";
			}
	}

	public static Action getAboutAction(final JFrame parent, final String abouttext)  {
		return new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				JReqTools.showAboutMessage(parent, abouttext);
				}
			};
			
	}

	static Border getFrameBorder()  {
		return border;
	}

	static int getFrameSpace()  {
		return frameSpace;
	}

	public static Icon getIcon(String name)  {
		Icon result = null;
		
		result = (Icon) icons.get(name);
		if (result != null) return result;
		
		result = UIManager.getIcon("jreqtools." + name + "Icon");
		if (result != null) return result;
		
		try
			{
			InputStream resource = Class.forName("jreqtools.JReqTools").getResourceAsStream("icons/" + name + ".gif");
			if (resource == null) return new XIcon(); 
		
			BufferedInputStream in = new BufferedInputStream(resource);
			ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
			byte [] buffer = new byte[1024];
			int n;
			while ((n = in.read(buffer)) > 0) out.write(buffer, 0, n);
			in.close();
			out.flush();
			buffer = out.toByteArray();
			result = new ImageIcon(buffer);
		
			icons.put(name, result);
			return result;
			}
		catch (Exception x ) { System.err.println(x); }
		
		return new XIcon();
	}

	static Insets getObjectInsets(int top, int left, int bottom, int right)  {
		return new Insets(
			top 		< 0 ? objectSpace : 0,
			left 		< 0 ? objectSpace : 0,
			bottom	< 0 ? objectSpace : 0,
			right	< 0 ? objectSpace : 0
			);
	}

	static int getObjectSpace()  {
		return objectSpace;
	}

	public static String getResourceString(String id)  {
		try
			{
			return resources.getString(id);
			}
		catch (MissingResourceException x) 
			{
			JReqTools.showMessage(x.getLocalizedMessage() + ":\n\n'" + id + "'" );
			return "MISSING RESOURCE: " + id;
			}
	}

	public static String getResourceString(String id, String arg1)  {
		String res = getResourceString(id);
		
		try
			{
			res = MessageFormat.format(res, new Object [] { arg1 });
			}
		catch (Exception x) { JReqTools.showException(x); }
		
		return res;
	}

	public static byte requestByte(JFrame parent, String title, String message, byte defaultValue)  {
		ByteRequester r = new ByteRequester(title, message, defaultValue);
		r.request(parent);
		return r.getByte();
	}

	public static double requestDouble(JFrame parent, String title, String message, double defaultValue)  {
		DoubleRequester r = new DoubleRequester(title, message, defaultValue);
		r.request(parent);
		return r.getDouble();
	}

	public static float requestFloat(JFrame parent, String title, String message, float defaultValue)  {
		FloatRequester r = new FloatRequester(title, message, defaultValue);
		r.request(parent);
		return r.getFloat();
	}

	public static int requestInteger(JFrame parent, String title, String message, int defaultValue)  {
		IntegerRequester r = new IntegerRequester(title, message, defaultValue);
		r.request(parent);
		return r.getInteger();
	}

	public static long requestLong(JFrame parent, String title, String message, long defaultValue)  {
		LongRequester r = new LongRequester(title, message, defaultValue);
		r.request(parent);
		return r.getLong();
	}

	public static Object requestObject(JFrame parent, String title, String message, Object [] selectableValues)  {
		return requestObject(parent, title, message, selectableValues, null);
	}

	public static Object requestObject(JFrame parent, String title, String message, Object [] selectableValues, Object selectedValue)  {
		SelectRequester r = new SelectRequester(title, message, selectableValues, selectedValue);
		if (r.request(parent) != SelectRequester.OK) return null;
		return r.getSelectedValue();
	}

	public static char [] requestPassword(JFrame parent, String title, String message)  {
		PasswordRequester r = new PasswordRequester(title, message);
		if (r.request(parent) != PasswordRequester.OK) return null;
		return r.getPassword();
	}

	public static short requestShort(JFrame parent, String title, String message, short defaultValue)  {
		ShortRequester r = new ShortRequester(title, message, defaultValue);
		r.request(parent);
		return r.getShort();
	}

	public static String requestString(JFrame parent, String title, String message, String defaultValue)  {
		StringRequester r = new StringRequester(title, message, defaultValue);
		if (r.request(parent) != StringRequester.OK) return null;
		return r.getString();
	}

	public static String requestText(JFrame parent, String title, String message, String defaultValue)  {
		TextRequester r = new TextRequester(title, message, defaultValue);
		if (r.request(parent) != TextRequester.OK) return null;
		return r.getText();
	}

	public static void showAboutMessage(String message)  {
		showAboutMessage(null, message);
	}

	public static void showAboutMessage(JFrame parent, String message)  {
		Dialog d = new MessageDialog(parent, getResourceString("About"), message);
		d.setVisible(true);
	}

	public static void showError(String message)  {
		showError(null, message);
	}

	public static void showError(JFrame parent, String message)  {
		Dialog d = new MessageDialog(parent, getResourceString("Error"), message);
		d.setVisible(true);
	}

	public static void showException(Throwable throwable)  {
		showException(null, throwable);
	}

	public static void showException(JFrame parent, Throwable throwable)  {
		Dialog d = new MessageDialog(parent, getResourceString("Exception"), createStringFromException(throwable));
		d.setVisible(true);
	}

	public static void showMessage(String message)  {
		showMessage(null, message);
	}

	public static void showMessage(JFrame parent, String message)  {
		Dialog d = new MessageDialog(parent, getResourceString("Message"), message);
		d.setVisible(true);
	}

	public static void showNotYetImplemented()  {
		showNotYetImplemented(null);
	}

	public static void showNotYetImplemented(JFrame parent)  {
		Dialog d = new MessageDialog(parent, getResourceString("Sorry"), getResourceString("NotYetImplemented"));
		d.setVisible(true);
	}

	public static boolean showQuestion(String message)  {
		return showQuestion(null, message, getResourceString("Yes"), getResourceString("No"));
	}

	public static boolean showQuestion(JFrame parent, String message)  {
		return showQuestion(parent, message, getResourceString("Yes"), getResourceString("No"));
	}

	public static boolean showQuestion(String message, String yesButton, String noButton)  {
		return showQuestion(null, message, yesButton, noButton);
	}

	public static boolean showQuestion(JFrame parent, String question, String yesButton, String noButton)  {
		YesNoDialog d = new YesNoDialog(parent, getResourceString("Question"), question, yesButton, noButton);
		d.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		d.setVisible(true);
		
		return d.getAnswer();
	}

}
