/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import jreqtools.gadgets.*;

/**
* This is a requester for look and feels. The chosen look and feel can also be set immediately.
* 
* @author	Markus Hillenbrand
*/

public class LookAndFeelRequester extends SelectRequester {
	private static String []	classnames		= null;
	private	Component		set				= null;
	
	public LookAndFeelRequester()  {
		this("JReqTools: LookAndFeelRequester", null);
	}

	public LookAndFeelRequester(String message)  {
		this("JReqTools: LookAndFeelRequester", message);
	}

	public LookAndFeelRequester(String title, String message)  {
		super(title, message, getInstalledLookAndFeels());
	}

	static {
		try
			{
			boolean found = false;
			
			Class c = Class.forName("swing.addon.plaf.threeD.ThreeDLookAndFeel");
			
			UIManager.LookAndFeelInfo [] lfs = UIManager.getInstalledLookAndFeels();
			for (int i=0; i<lfs.length; ++i) if (lfs[i].getClassName().equals("swing.addon.plaf.threeD.ThreeDLookAndFeel")) found = true;
			
			if (! found) UIManager.installLookAndFeel("3D Look & Feel", "swing.addon.plaf.threeD.ThreeDLookAndFeel");
			}
		catch (ClassNotFoundException x) { }
		
		try
			{
			boolean found = false;
			
			Class c = Class.forName("swing.addon.plaf.themelf.ThemeLookAndFeel");
			
			UIManager.LookAndFeelInfo [] lfs = UIManager.getInstalledLookAndFeels();
			for (int i=0; i<lfs.length; ++i) if (lfs[i].getClassName().equals("swing.addon.plaf.themelf.ThemeLookAndFeel")) found = true;
			
			if (! found) UIManager.installLookAndFeel("ThemeLF", "swing.addon.plaf.themelf.ThemeLookAndFeel");
			}
		catch (ClassNotFoundException x) { }
		
		try
			{
			boolean found = false;
			
			Class c = Class.forName("de.hillenbrand.plaf.win.WinLookAndFeel");
			
			UIManager.LookAndFeelInfo [] lfs = UIManager.getInstalledLookAndFeels();
			for (int i=0; i<lfs.length; ++i) if (lfs[i].getClassName().equals("de.hillenbrand.plaf.win.WinLookAndFeel")) found = true;
			
			if (! found) UIManager.installLookAndFeel("Win", "de.hillenbrand.plaf.win.WinLookAndFeel");
			}
		catch (ClassNotFoundException x) { }
	}

	public static String [] getInstalledLookAndFeels()  {
		UIManager.LookAndFeelInfo [] info = UIManager.getInstalledLookAndFeels();
		
		String [] result = new String [info.length];
		classnames = new String[info.length];
		
		for (int i=0; i<info.length; i++)
			{
			result[i] = info[i].getName();
			classnames[i] = info[i].getClassName();
			}
		
		return result;
	}

	public String getLookAndFeel()  {
		String selected = (String)getSelectedValue();
		
		String [] all = getInstalledLookAndFeels();
		for (int i=0; i<all.length; ++i) 
			{
			if (all[i].equals(selected)) return classnames[i];
			}
		
		return null;
	}

	protected int getReturnValue()  {
		int result = super.getReturnValue();
		
		if (result == OK && set != null)
			{
			try
				{
				UIManager.setLookAndFeel(getLookAndFeel());
				SwingUtilities.updateComponentTreeUI(SwingUtilities.getRoot(set));
				}
			catch (UnsupportedLookAndFeelException un)
				 {
				 JReqTools.showError(JReqTools.getResourceString("Error.01", getLookAndFeel())); 
				 }
			catch (Exception x) 
				{ 
				System.out.println(x); 
				}
			}
		
		return result;
	}

	public void setAfterSelection(Component tree)  {
		set = tree;
	}

}
