/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import jreqtools.gadgets.*;

/**
* This is a requester for a list of strings. The user can select one of them.
* 
* @author	Markus Hillenbrand
*/

public class SelectRequester extends AbstractRequester {
	private	Object []		objects		= null;
	private	Object			selected	= null;
	private String			message	= null;
	private	int 				answer		= CANCEL;
	
	private class Dialog extends JDialog {
		private JList	 	list		 	= new JList(objects);
		private	JScrollPane	scrollpane	= new JScrollPane(list);
		
		public Dialog(JFrame parent)  {
			super(parent, "JReqTools: SelectRequester", true);
			
			// Gadgets einrichten
			scrollpane.setPreferredSize(new java.awt.Dimension(200,200));
			list.setSelectedValue(selected, true);
			list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			if (objects != null && list.getSelectedValue() == null) list.setSelectedIndex(0);
			
			// Listener
			list.addMouseListener(new MouseAdapter() { public void mouseClicked(MouseEvent e) { if (e.getClickCount() > 1) ok(); } });
			list.addKeyListener(new KeyAdapter() { public void keyTyped(KeyEvent e) { 
				if (e.getKeyChar() == KeyEvent.VK_ENTER) ok(); 
				else if (e.getKeyChar() == KeyEvent.VK_ESCAPE) cancel(); } });
			
			// Buttons erzeugen
			OkCancelButtons buttons = new OkCancelButtons(SelectRequester.this,
				new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); } },
				new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); } },
				listener);
			if (objects == null) buttons.getOkButton().setEnabled(false);
			
			// Panel erzeugen
			JPanel p = new YFramePanel();
			p.add(new MessagePanel(message, scrollpane));
			p.add(new Strut());
			p.add(buttons);
			
			// Alles hinzufgen
			setContentPane(p);
			getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
			pack();
		}

		private void cancel()  {
			answer = CANCEL;
			setVisible(false);
		}

		private void ok()  {
			answer = OK;
			selected = list.getSelectedValue();
			setVisible(false);
		}

		public void setVisible(boolean b)  {
			if (b) list.requestFocus();
			super.setVisible(b);
		}

	}

	/**
	* Constructs a new StringRequester.
	*/

	public SelectRequester()  {
		this("JReqTools: SelectRequester", null, null, null);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	defaultstring - The initial value of the text field.
	*/

	public SelectRequester(Object [] data)  {
		this("JReqTools: SelectRequester", null, data, null);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public SelectRequester(String message, Object [] data)  {
		this("JReqTools: SelectRequester", message, data, null);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public SelectRequester(String message, Object [] data, Object selected)  {
		this("JReqTools: SelectRequester", message, data, selected);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public SelectRequester(String title, String message, Object [] data)  {
		this(title, message, data, null);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public SelectRequester(String title, String message, Object [] data, Object selected)  {
		setTitle(title);
		setMessage(message);
		setValues(data);
		setSelectedValue(selected);
	}

	protected javax.swing.JDialog createJDialog(JFrame parent)  {
		return new Dialog(parent);
	}

	protected int getReturnValue()  {
		return answer;
	}

	public Object getSelectedValue()  {
		return selected;
	}

	/**
	* Sets the message to be shown in the requester.
	*/

	public void setMessage(String newmessage)  {
		message = newmessage;
	}

	public void setSelectedValue(Object object)  {
		selected = object;
	}

	/**
	* Sets the string in the requester.
	*/

	public void setValues(Object [] data)  {
		objects = data;
	}

}
